# browserify-shim [![build status](https://secure.travis-ci.org/thlorenz/browserify-shim.png)](http://travis-ci.org/thlorenz/browserify-shim)

Shims modules that aren't installed as `npm` modules so they can be browserified even if they aren't commonJS
compatible.

```js
var browserify = require('browserify')
  , shim = require('browserify-shim');

var bundled = browserify({ debug: true })

    // jquery attaches itself to the window as '$' so we assign the exports accordingly
  .use(shim({ alias: 'jquery', path: './js/vendor/jquery.js', exports: '$' }))

    // underscore is commonJS compliant, so no further export is needed which we specify by assigning exports 'null'
  .use(shim({ alias: 'underscore', path: './js/vendor/underscore.js', exports: null }))

  .addEntry('./js/entry.js')
  .bundle();

fs.writeFileSync(builtFile, bundled);
```

## Installation

    npm install browserify-shim

## Features

- shims **non commonJS** modules in order for them to be **browserified** by specifying an alias, the path to the file and
  the identifier under which the module attaches itself to the global `window` object
- handles even those modules out there that just declare a `var foo = ...` on the script level and assume it gets attached to the
  `window` object since the only way they will ever be run is in the global context - "ahem, ... NO?!"
- loads commonJS modules that are not residing in your `node_modules` from a specific path
- includes `depends` to support shimming libraries that depend on other libraries to be in the global namespace

## Dependents

Some libraries depend on other libraries to have attached their exports to the window.

As an example [backbone.stickit](http://nytimes.github.com/backbone.stickit/) depends on Backbone, underscore.js,
and jQuery or Zepto.

We would properly declare its dependents when shimming it as follows:
```js

var jquery     =  { alias: 'jquery'     ,  path: './js/vendor/jquery.js'     ,  exports: '$' }
  , underscore =  { alias: 'underscore' ,  path: './js/vendor/underscore.js' ,  exports: null }
  , backbone   =  { alias: 'backbone'   ,  path: './js/vendor/backbone.js'   ,  exports: null };

var bundled = browserify()
  .use(shim(jquery))
  .use(shim(underscore))
  .use(shim(backbone))
  .use(shim({ 
      alias: 'backbone.stickit'
    , path: './js/vendor/backbone.stickit.js'
    , exports: null
    , depends: [ jquery, underscore, backbone ]  
    })
  )
  .addEntry('./js/entry.js')
  .bundle();
```

**Note:** the order of shim declarations doesn't matter, i.e. we could have shimmed `backbone.stickit` at the very top
(before the libraries it depends on).

If a shim depends on only one other shim you can pass just that (instead of an `Array`), e.g.: 

```js
var underscore =  { alias: 'underscore', path: './js/vendor/underscore.js', exports: null }

[..]
  .use(shim({ alias: 'backbone', path: './js/vendor/backbone.js', exports: null, depends: underscore })
[..]
```

## Examples

- [shim-jquery](https://github.com/thlorenz/browserify-shim/tree/master/examples/shim-jquery)
- [shim-underscore](https://github.com/thlorenz/browserify-shim/tree/master/examples/shim-underscore)
